/*!
	@file: optimization/FileLog.h
	@author: Matthias Wacker
*/

#ifndef _PARAMLOG_H_
#define _PARAMLOG_H_

#include <string>
#include <fstream>
#include "core/optimization/blackbox/OptLogBase.h"

namespace OPTIMIZATION
{
    
  class ParamLog : public OptLogBase
  {

    public:
    
    /*!
      default
    */
    ParamLog(){};

    /*!
      @param file string with the filename to log to
    */
    //FileLog( string file);
    ParamLog( std::string file);

    /*!
      destructor
    */
    virtual ~ParamLog();

    /**! Write error message to an output device (file, stdio, etc.)
    */
    virtual void writeLogError(const char* szMessage);

    /**! Write warning message to an output device (file, stdio, etc.)
    */
    virtual void writeLogWarning(const char* szMessage);

    /**! Write trace message to an output device (file, stdio, etc.)
    */
    virtual void writeLogTrace(const char* szMessage);

    /**! Write parameter vector to output device (file, stdio, etc.)
    *
    * @param parammatrix parameter matrix
    */
    virtual void writeParamsToFile(OPTIMIZATION::matrix_type& parammatrix);

    private:
    
    std::ofstream m_stream;
  };//class
}//namespace

#endif /* _PARAMLOG_H_ */
